@@ -1,18 +1,44 @@
-# -*- coding: utf-8 -*-
+#!/usr/bin/python
+# coding: utf-8
+# =============================================================================
+# Author: Denis Wallerich
+# Email: denis.wallerich@datim.fr
+# Date: 2012-11-10
+# Copyright: This module has been placed in the public domain.
+#
+# Ibm cobol (sql/cics/dli embedded) pygments Lexer
+#
+# ========== ==================================================================
+# 2012-11-16 - Fix : float regex before integer
+#            - Add inline-commentaire '*>' (not the IBM default)
+#            - Change cics/dli keywords color...
+#            - Extend CICS_KEYWORDS
+#            - Add EBCDIC features:
+#              automatic detection of EBCDIC encoding (cp500,cp1140,...) 
+#	       also convert the raw binary input string in fixed utf-8 lines
+#              "lrecl=" option can be passed (80 is the default source IBM lrecl)
+#              if not EBCDIC encoding detected, all lines are justified to
+#              'lrecl=80' value (enforce existence of "MargeRight" tokens)
+#	       Remarks : "chardet/guess" encoding do not detect EBCDIC chart
+# ========== ==================================================================
+# $TODO      cobol-picture token regex
+#            separate dli/cics/db2 commands/builtins and keywords
+#            performances 
+# ========== ==================================================================
+
 import re
 from pygments.lexer import RegexLexer, include, bygroups, using, this
 from pygments.token import Text, Other, Whitespace, Keyword, Name, Comment, String, \
                            Error, Number, Operator, Generic, Punctuation
 from pygments.style import Style
+from pygments.util import get_int_opt
 
 class IBMCOBOLStyle(Style):
     default_style = ""
     background_color = "#111111"
     styles = {
-        
         Error:                  'bg:#ff0000 #ffffff',
         Text.Tag:               'bg:#9609e9 border:#777777 #ffffff ',
-
         # custom tags:
         Text.Scrubol:           'italic #555555',
         Text.ScrubolTag:        'bg:#444444 border:#777777 #ffffff ',
@@ -20,40 +46,39 @@ class IBMCOBOLStyle(Style):
         Text.ScrubolTagDoc:     'bg:#444444 border:#777777 #ffffff ',
         Text.ScrubolWarn:       'italic #555555',
         Text.ScrubolTagWarn:    'bg:#ff0000 border:#ff0000 #ffffff ',
-
+        # 
         Comment.MargeLeft:      '#fcff70', 
         Comment.MargeRight:     '#fcff70', 
         Comment:                '#00c8ff',
         Comment.Debug:          '#0000c9',
         Comment.Continuation:   '#9609e9',
-
+        # cics
         Comment.CicsExec:       '#9609e9',
         Name.Cics:              '#fc9b00',
-        Keyword.Cics:           '#fc6b00',
+        Keyword.Cics:           '#fc3000',
         Punctuation.Cics:       '#b1f603',
         Comment.AllCics:        '#901060',
         Operator.Cics:          '#00d300',
-
+        # dli 
         Comment.DliExec:        '#9609e9',
         Name.Dli:               '#fc9b00',
-        Keyword.Dli:            '#fc6b00',
+        Keyword.Dli:            '#fc3000',
         Punctuation.Dli:        '#b1f603',
         Comment.AllDli:         '#901060',
         Operator.Dli:           '#00d300',
-            
+        # db2    
         Comment.SqlExec:        '#9609e9',
         Comment.AllSql:         '#900060',
         Keyword.Sql:            '#933493',
         Name.Sql:               '#d898f4',
         Punctuation.Sql:        '#b1f603',
         Operator.Sql:           '#933493',
-
+        # cobol
         Keyword:                '#ff0000',
         Name.Division:          'underline bold #ff0000',
         Name.CompileOpts:       'bold #ff0000',
         Name.Section:           'underline #ff0000',
         Name.Paragraphe:        'underline #00d300',
-
         Name:                   '#00d300',
         Name.Function:          '#900090',
         Punctuation:            '#b1f603',
@@ -62,6 +87,7 @@ class IBMCOBOLStyle(Style):
         String:                 '#ffffff',
     }
 
+# DB2V9 keywords
 DB2_KEYWORDS=[
     "ABSOLUTE","ACCESS","ACTION","ACTIVATE","ADA","ADD","AFTER","ALIAS","ALL","ALLOCATE",
     "ALLOW","ALTER","ALTERIN","ALWAYS","AND","ANY","APPEND","ARE","AS","ASC","ASCII",
@@ -134,30 +160,51 @@ DB2_KEYWORDS=[
     "WHEN","WHENEVER","WHERE","WHILE","WITH","WITHOUT","WORK","WRAPPER","WRITE","YEAR",
     "YEARS","YES","ZONE"]
 
+
 DLI_KEYWORDS=[
     "ACCEPT","DLET","GN","GET","NEXT","GNP","IN","PARENT","GU","UNIQUE","ISRT","INSERT",
     "POS","POSITION","REPL","REPLACE","RETRIEVE","SCHD","SCHEDULE","TERM","TERMINATE",
     "CHKP","CHECKPOINT","DEQ","LOAD","LOG","QUERY","REFRESH","ROLB","ROLL","ROLS","SETS","SETU",
     "STAT","SYMCHKP","XRST"]
 
-CICS_KEYWORDS=[
-    "ABCODE","ABEND","ADDRESS","AID","ALARM","ASKTIME","ASSIGN","CANCEL",    
-    "COMMAREA","CURSOR","CWA","START","DATA","DATASET","DELETE","DELETEQ","DUMP",    
-    "DUMPCODE","DUPKEY","ENDBR","ENDFILE","EQUAL","ERASE","ERROR","FILE",    
-    "FLENGTH","FORMATTIME","FREEKB","FREEMAIN","FROM","GENERIC","GETMAIN","GTEQ",    
-    "HANDLE","CONDITION","HOLD","IGNORE","INITIMG","INTERVAL","INTO","ISSUE_DISCONNECT",
-    "ITEM","ITEMERR","KEYLENGTH","LABEL","LENGERR","LENGTH","OF","LINK","LOAD","MAP",
-    "MAPFAIL","MAPSET","NOHANDLE","NOTFND","PROGRAM","QIDERR","QUEUE","RBA","READ",
-    "READNEXT","READPREV","READQ_TS","RECEIVE","RETURN","REWRITE","RIDFLD","SEND","TEXT",    
-    "SET","SHARED","STARTBR","SYNCPOINT","TERMID","TRANSID","UPDATE","USERID","CONDITION",
-    "CURSOR","DISCONNECT","ENDBR","ENDFILE","EQUAL","ERASE","ERROR","FLENGTH","FORMATTIME",
-    "FREEKB","FREEMAIN","FROM","GENERIC","GETMAIN","GTEQ","HANDLE","HOLD","IGNORE",    
-    "INITIMG","INTERVAL","INTO","ISSUE","ITEM","ITEMERR","KEYLENGTH","LABEL","LENGERR",    
-    "LENGTH","LINK","LOAD","MAP","MAPFAIL","MAPSET","NOHANDLE","NOTFND",    
-    "OF","PROGRAM","QIDERR","QUEUE","RBA","READ","READNEXT","READPREV","READQ",    
-    "RECEIVE","RETURN","REWRITE","REWRITE","RIDFLD","ROLLBACK","SEND","SEND","SET",    
-    "SHARED","START","STARTBR","SYNCPOINT","TERMID","TEXT","TRANSID","TS",    
-    "UPDATE","USERID","WRITE","WRITEQ","XCTL"]
+CICS_KEYWORDS=["DATASET","NOHANDLE","OF",
+"DEFAULT","EOC","EOF","ABCODE","ABEND","ABORT","ABSTIME","ACCOUNT","ACCUM","ACTIVATE","ACTPARTN","ADD","ADDRESS",
+"AID","ALARM","ALL","ALLOCATE","ALTSCRN","ANY_CICS","ANYKEY","APPLID","ASIS","ASKIP","ASKTIME","ASM","ASSIGN","ATTACH",
+"ATTACHID","ATTRB","AUTO","AUTOPAGE","AUXILIARY","BANG","BAR","BASE","BIF","BIT_AND","BLANK","BLINK","BLOCK","BOTTOM","BOX","BRT",
+"BTRANS","BUFFER","BUFSZE","BUILD","CANCEL","CARD","CASE","CBIDERR","CBUFF","CCERROR","CHAR_STRING","CHARSZE","CICS","CICS_ID",
+"CICSLETTER","CLEAR","CLRPARTN","CNOTCOMPL","COBOL","COLON","COLOR","COLUMN","COMMA","COMMAREA","COMPLETE","CONDITION","CONFIRM",
+"CONFIRMATION","CONNECT","CONSOLE","CONTROL","CONVERSE","CONVID","COPY","CSA","CTLCHAR","CTRL","CURRENT","CURSOR","CWA","CWALENG",
+"DATA","DATAONLY","DATASTR","DATE","DATEFORM","DATESEP","DAYCOUNT","DAYOFMONTH","DAYOFWEEK","DCT","DDMMYY","DEBKEY","DEBREC","DEEDIT",
+"DEFAULT","DEFINE","DEFRESP","DELAY","DELETE","DELETEQ","DELIMITER","DEQ","DEST","DESTCOUNT","DESTID","DESTIDLENG","DET","DFHMDF","DFHMDI",
+"DFHMSD","DFHPDI","DFHPSD","DIGIT","DISABLED","DISCONNECT","DOT","DRK","DSATTS","DSECT","DSIDERR","DSSTAT","DUMP","DUMPCODE","DUPKEY","DUPREC",
+"ECADDR","EI","EIB","END","ENDBR","ENDDATA","ENDEXEC","ENDFILE","ENDINPT","ENDOUTPUT","ENQ","ENQBUSY","ENTER","ENTRY","ENTRYNAME","ENVDEFERR",
+"EOC","EODS","EOF","EQ","EQUAL","ERASE","ERASEAUP","ERROR","ERRTERM","EVENT","EXEC","EXPIRED","EXPONENT","EXTATT","EXTDS","EXTRACT","FACILITY",
+"FCI","FCT","FIELD","FIELDS","FILE","FINAL","FIRST","FLDSEP","FLENGTH","FMH","FMHPARM","FORMATTIME","FORMFEED","FREE","FREEKB","FREEMAIN",
+"FROM","FROMLENGTH","FRSET","FSET","FULL","FUNCERR","GCHARS","GCODES","GENERIC","GETMAIN","GRPNAME","GT","GTEQ","HANDLE","HEADER","HEXCHAR",
+"HILIGHT","HOLD","HONEOM","HOST_ID","HTAB","IC","IGNORE","IGREQCD","IGREQID","ILLOGIC","IN","IN_MULTI_LINE_COMMENT","IN_SINGLE_LINE_COMMENT",
+"INBFMH","INITIAL","INITIMG","INOUT","INPARTN","INTERVAL","INTO","INVERRTERM","INVITE","INVLDC","INVMPSZ","INVPARTN","INVPARTNSET","INVREQ",
+"INVTSREQ","IOERR","ISCINVREQ","ISSUE","ITEM","ITEMERR","IUTYPE","JIDERR","JOURNAL","JUSFIRST","JUSLAST","JUSTIFY","KATAKANA","KEYLENGTH",
+"KEYNUMBER","L40","L64","L80","LABEL","LANG","LAST","LBRACKET","LDC","LDCMNEM","LDCNUM","LEAVEKB","LEFT","LENGERR","LENGTH","LETTER","LIGHTPEN",
+"LINE","LINEADDR","LINK","LIST","LOAD","LOGONMSG","LPAREN","LT","LUNAME","LUTYPE62","MAIN","MAP","MAPATTS","MAPCOLUMN","MAPERROR","MAPFAIL",
+"MAPHEIGHT","MAPLINE","MAPONLY","MAPPED","MAPSET","MAPSFX","MAPWIDTH","MASSINSERT","MAXLENGTH","MESSAGE","MINIMUM","MINUS","MIXED","MMDDYY",
+"MODE","MONITOR","MONTHOFYEAR","MSR","MSRCONTROL","MULTI_LINE_COMMENT","MUSTENTER","MUSTFILL","NAMEERROR","NETNAME","NEXT","NLEOM","NO",
+"NOAUTOPAGE","NOCHECK","NOEDIT","NOJBUFSP","NOMAPPING","NONVAL","NOPASSBKRD","NOPASSBKWR","NOQUEUE","NORM","NOSPACE","NOSPOOL","NOSTART",
+"NOSTG","NOSUSPEND","NOTALLOC","NOTAUTH","NOTE","NOTFND","NOTOPEN","NOTRUNCATE","NOWAIT","NUM","NUM_CONSTANT","NUMITEMS","NUMREC","NUMTAB",
+"OBFMT","OCCURS","OFF","ON","OPCLASS","OPERID","OPERKEYS","OPERPURGE","OPID","OPSECURITY","OUT","OUTLINE","OUTPARTN","OVER","OVERFLOW",
+"PAGE","PAGENUM","PAGING","PARTN","PARTNFAIL","PARTNPAGE","PARTNS","PARTNSET","PASS","PASSBK","PCT","PERCENT","PERFORM","PGMIDERR",
+"PICIN","PICOUT","PIPLENGTH","PIPLIST","PLI","PLUS","POINT","POP","POS","POST","PPT","PREPARE","PRINSYSID","PRINT","PROCESS",
+"PROCLENGTH","PROCNAME","PROFILE","PROGRAM","PROT","PROTECT","PS","PSEUDOBIN","PURGE","PUSH","QBUSY","QIDERR","QNAME","QUERY","QUESTION",
+"QUEUE","QZERO","RBA","RBRACKET","RDATT","READ","READNEXT","READPREV","READQ","RECEIVE","RECFM","RELEASE","REPLACE",
+"REQID","RESET","RESETBR","RESOURCE","RESTART","RETAIN","RETPAGE","RETRIEVE","RETURN","REVERSE","REWRITE","RIDFLD","RIGHT",
+"ROLLBACK","ROLLEDBACK","ROUTE","RPAREN","RPG","RPROCESS","RRESOURCE","RRN","RTEFAIL","RTERMID","RTESOME","RTRANSID",
+"SAME","SCRNHT","SCRNWD","SELNERR","SEMICOLON","SEND","SESSBUSY","SESSION","SESSIONERR","SET","SHARED","SIGDATA",
+"SIGNAL","SINGLE","SINGLE_LINE_COMMENT","SIT","SIZE","SLASH","SOSI","STANDARD","STAR","START","STARTBR","STARTCODE",
+"STATIONID","STORAGE","STRFIELD","SUBADDR","SUFFIX","SUSPEND","SYNCLEVEL","SYNCPOINT","SYSBUSY","SYSID","SYSIDERR","SYSTEM",
+"TABLES","TASK","TCT","TCTUA","TCTUALENG","TD","TELLERID","TERM","TERMCODE","TERMERR","TERMID","TERMIDERR",
+"TERMINAL","TEXT","TIME","TIMESEP","TIOAPFX","TITLE","TOLENGTH","TRACE","TRACEID","TRAILER","TRANSID","TRANSIDERR",
+"TRANSP","TRIGGER","TS","TSIOERR","TWA","TWALENG","TYPE","UNATTEND","UNDER","UNDERLINE","UNEXPIN","UNLOCK","UNPROT","UPDATE","USER",
+"USERID","VALIDATION","VALIDN","VIEWPOS","VIEWSZE","VOLUME","VOLUMELENG","VTAB","WAIT","WPMEDIA1","WPMEDIA2","WPMEDIA3","WPMEDIA4",
+"WRBRK","WRITE","WRITEQ","WRONGSTAT","XCTL","XINIT","XOR","YEAR","YES","YYDDD","YYDDMM","YYMMDD","ZERO"]
 
 COBOL_KEYWORDS=[
     "ACCEPT","ACCESS","ADD","ADDRESS","ADVANCING","AFTER","ALL","ALPHABET","ALPHABETIC",
@@ -171,7 +218,7 @@ COBOL_KEYWORDS=[
     "COPY","CORR","CORRESPONDING","COUNT","CURRENCY","DATA","DATE","DATE-COMPILED",
     "DATE-WRITTEN","DAY","DAY-OF-WEEK","DBCS","DEBUGGING","DECIMAL-POINT","DECLARATIVES",
     "DELETE","DELIMITED","DELIMITER","DEPENDING","DESCENDING","DISPLAY","DISPLAY-1",
-    "DIVIDE","DIVISION","DOWN","DUPLICATES","DYNAMIC","EBCDIC","EGCS","EJECT","ELSE",
+    "DIVIDE","DIVISION","DOWN","DUPLICATES","DYNAMIC","EBCDIC","EGCS","ELSE",
     "END","END-ADD","END-CALL","END-COMPUTE","END-DELETE","END-DIVIDE","END-EVALUATE",
     "END-IF","END-MULTIPLY","END-OF-PAGE","END-PERFORM","END-READ","END-RETURN",
     "END-REWRITE","END-SEARCH","END-START","END-STRING","END-SUBTRACT","END-UNSTRING",
@@ -194,7 +241,7 @@ COBOL_KEYWORDS=[
     "REPLACING","RERUN","RESERVE","RETURN","RETURN-CODE","RETURNING","REVERSED",
     "REWIND","REWRITE","RIGHT","ROUNDED","RUN","SAME","SD","SEARCH","SECTION","SECURITY",
     "SEGMENT-LIMIT","SELECT","SENTENCE","SEPARATE","SEQUENCE","SEQUENTIAL",
-    "SET","SIGN","SIZE","SKIP1","SKIP2","SKIP3","SORT","SORT-MERGE","SOURCE-COMPUTER",
+    "SET","SIGN","SIZE","SORT","SORT-MERGE","SOURCE-COMPUTER",
     "SPACE","SPACES","SPECIAL-NAMES","STANDARD","STANDARD-1","STANDARD-2","START",
     "STATUS","STOP","STRING","SUBTRACT","SUPPRESS","SYMBOLIC","SYNC","SYNCHRONIZED",
     "TALLYING","TAPE","TEST","THAN","THEN","THROUGH","THRU","TIME","TIMES","TO","TOP",
@@ -237,32 +284,77 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
 
    CBL/PROCESS statement can start in columns 1 through 70
    Code the PROCESS statement before the IDENTIFICATION DIVISION header and before
-   any comment lines or compiler-directing statements. """
-    
+   any comment lines or compiler-directing statements.
+
+ SOME EBCDIC CODEPAGES:
+ ---------------------
+    cp037  : Australie, Brésil, Canada, Nouvelle Zélande, Portugal, Afrique du Sud, USA
+    cp297  : EBCDIC "France"
+    cp500  : francophones (suisses, belges, canadiennes...)
+    cp819  : ISO-8859-1 (Latin 1),
+    cp1047 : cp037 utilisé dans les services UNIX des S/390 et z/OS, et sous VM/ESA.
+    cp1147 : cp297 + Euro sign
+    cp1148 : cp500 + Euro sign
+"""
+
     def __init__(self, **options):
         RegexLexer.__init__(self, **options)
+        self.lrecl=get_int_opt(options,'lrecl', default=80)
         self.stripnl=False
         self.stripall=False
+        self.tabsize=0
+        if self.encoding == 'cp1147':
+            import cp1147
+        elif self.encoding == 'chardet':
+            self.encoding = 'guess'  # chardet it's not efficient with EBCDIC
+        
+            
+    def get_tokens_unprocessed(self, text):
+        if  text[:1024].count('\x40') > text[:1024].count('\x20'): # EBCDIC test (it's suffisant for text)
+            if self.encoding in ('guess','latin1'):   
+                self.encoding='cp500' # EBCDIC Latin-1
+                text=text.encode('latin1').decode(self.encoding)
+            else:
+                text=text.encode(self.encoding).decode('cp500') 
+                self.encoding='cp500' # EBCDIC Latin-1
+        if (self.encoding <> 'guess' and        
+            ' 012ABC'.encode(self.encoding) == "\x40\xf0\xf1\xf2\xc1\xc2\xc3"):
+            #good EBCDIC self.encoding
+            if self.ensurenl:
+                text_len=len(text) - 1 # since ensurenl add '\n'
+            else:
+                text_len=len(text)
+            if text_len % self.lrecl == 0:  # to preserve against "MVS variable length record"
+                text=''.join([text[i:i+self.lrecl] + '\n'  # split block in lines
+                             for i in range(0, text_len, self.lrecl)])
+            else:
+                raise ValueError("inconsistent textlen %i and lrecl %i (varying record length ?)"
+                                 % (text_len,self.lrecl))
+        else:
+            #ASCII:
+            text=''.join([l.ljust(self.lrecl) + '\n'
+                         for l in text.splitlines()]) # enforce 80 columns
+        
+        for item in RegexLexer.get_tokens_unprocessed(self,text): 
+            yield item
+       
 
     name = 'ibmcobol'
     aliases = ['cobol','ibmcob','ibm_cob', 'IBM_COBOL']
     filenames = ['*.cbl','*.CBL','*.cob','*.COB']
     mimetypes = ["text/x-cobol"]
-    flags = re.IGNORECASE | re.DOTALL | re.MULTILINE 
-
+    flags = re.IGNORECASE | re.DOTALL | re.MULTILINE
+      
     start_boundarie=r"(?<![-_])\b"
     end_boundarie  =r"(?=\b[^-_])"
-    
     # $TODO merge this ! ( due to ambiguity issue with numbers in sdf2 grammar)
     cobol_name=  r'(([0-9]+[\-\_][0-9\-\_]*[A-Za-z][A-Za-z0-9\-\_]*[A-Za-z0-9])' +\
                  r'|([0-9]+[\-][A-Za-z0-9\-\_]*)'                                +\
                  r'|([0-9]*[A-Za-z][A-Za-z0-9\-\_]*[A-Za-z0-9]*))'       
-
     #(?!.{0,7}\n) => no match in right margin 
     #(?<=\n.{7})  => match after left margin
 
     custom_tag = "MyTag"
-
     # My custom tags in left margin
     custom_marge_tag0 = r"\(\^[^_^w]\^\)"
     custom_marge_tag1 = r"\(\^_\^\)"
@@ -271,6 +363,7 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
     tokens = {
      'root': [
                include('special-in-cobol-tag'),
+               include('inline-comments'),
                (r"(^\s*(CBL|PROCESS)[^\n]*)",  Name.CompileOpts) ,
                (r"(^.{6})([\*\/].{65})(.{8})",
                  bygroups(Comment.MargeLeft, Comment, Comment.MargeRight)),
@@ -281,11 +374,11 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
              ],
 
     'cobol-content': [
-        #include('special-in-cobol-tag'),
                  (r'[^\n]{1,8}\n',   Comment.MargeRight, '#pop'),
                  (r'EXEC\s*SQL\s*',  Comment.SqlExec, 'sql-content'),
                  (r'EXEC\s*CICS\s*', Comment.SqlExec,'cics-content'),
                  (r'EXEC\s*DLI\s*',  Comment.SqlExec, 'dli-content'),
+                 include('inline-comments'),
                  include('sec-div'),
                  include('paragraphe'),
                  include('intrinsics'),
@@ -294,50 +387,47 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
                  include('keywords'),
                  include('variable'),
                  include('numbers'), # numbers after variables !
-                 include('operator'), # operators after variables ! 
+                 include('operator'), # operators after variables !
                  (r'[ ]+', Text)
                 ],
 
       'sec-div':[
-                     (r'(DECLARATIVES|END\s+DECLARATIVES)', Name.Division),
-                     (r'([a-zA-Z0-9-_]+\s+DIVISION)', Name.Division),
-                     (r'([a-zA-Z0-9-_]+\s+SECTION)', Name.Section)
-                     ],
+                 (r'(DECLARATIVES|END)(\s+)(DECLARATIVES)', bygroups(Name.Division,Text,Name.Division)),
+                 (r'([a-zA-Z0-9-_]+)(\s+)(DIVISION)', bygroups(Name.Division,Text,Name.Division)),
+                 (r'([a-zA-Z0-9-_]+)(\s+)(SECTION)', bygroups(Name.Section,Text,Name.Section))
+                 ],
 
       'paragraphe':[
-                     (r'((?<=\n.{7})(?!(SKIP.? |EJECT |EXEC |COPY ))[a-zA-Z0-9-_]+)(?=\s*[.])',
-                         Name.Paragraphe)
-                    ],
+                 (r'((?<=\n.{7})(?!(SKIP.? |EJECT |EXEC |COPY ))[a-zA-Z0-9-_]+)(?=\s*[.])',
+                      Name.Paragraphe)
+                   ],
  
       'keywords':[
-           ( start_boundarie + '(' + '|'.join(COBOL_KEYWORDS) + ')' + end_boundarie , Keyword),
+            (start_boundarie + '(' + '|'.join(COBOL_KEYWORDS) + ')' + end_boundarie , Keyword),
             (r'([\=\<\>])',Keyword) 
-            ],
+           ],
 
-      'variable': [
-            (cobol_name,Name) 
-            ],
+      'variable': [(cobol_name,Name)],
 
-      'operator': [(r'((?!.{0,7}\n)[\/\+\-\*])',Operator) ],
+      'operator': [(r'((?!.{0,7}\n)[\/\+\-\*])',Operator)],
 
-      'ponctuation': [
-            (r'([~&\^#\|\[\]`!@;,\.():])',Punctuation) ],
+      'ponctuation': [(r'([~&\^#\|\[\]`!@;,\.():])',Punctuation)],
  
       'intrinsics': [
-          (r'FUNCTION\s*' + start_boundarie +
-            '(' + '|'.join(COBOL_INTRINSICS) + ')' + end_boundarie, Name.Function)
+             (r'FUNCTION\s*' + start_boundarie +
+               '(' + '|'.join(COBOL_INTRINSICS) + ')' + end_boundarie, Name.Function)
             ],
 
       'strings': [
              (r"(\b[xXzZ])?'[^'\\\n]*(\\.[^'\\\n]*)*'?", String.Single),
              (r'(\b[xXzZ])?"[^"\\\n]*(\\.[^"\\\n]*)*"?', String.Double),
-           ],
+            ],
 
       'numbers': [
-             (r'[+-]?\d+(?![A-Z-_])', Number),
              (r'[+-]?\d*\.\d+([eE][-+]?\d+)?', Number),
-             (r'[+-]?\d+\.\d*([eE][-+]?\d+)?', Number)
-           ],       
+             (r'[+-]?\d+\.\d*([eE][-+]?\d+)?', Number),
+             (r'[+-]?\d+(?![A-Z-_])', Number),
+            ],       
 
      'dli-content': [
             include('special-in-exec-tag'),
@@ -349,8 +439,7 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
             include('strings'),
             (cobol_name,Name.Dli), 
             (r'[;:()\[\],\.]', Punctuation.Dli),
-
-        ],
+           ],
 
      'cics-content': [
             include('special-in-exec-tag'),
@@ -362,7 +451,7 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
             include('strings'),
             (cobol_name,Name.Cics), 
             (r'[;:()\[\],\.]', Punctuation.Cics),
-        ],
+           ],
      
      'sql-content': [
             include('special-in-exec-tag'),
@@ -378,12 +467,16 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
                ],
 
    'comments': [
+            include('inline-comments'),
             (r'(?<=\n).{6}', Comment.MargeLeft),
             (r'[^\n]{0,8}\n', Comment.MargeRight),
             (r'(?<=\n.{6})\*.*?\n', Comment),
             (r'\s+', Text)
               ],    
 
+   'inline-comments': [
+            (r"\*\>.*?$", Comment),
+            (r"(SKIP.?|EJECT)", Comment) ],    
 
    'special-in-cobol-tag': [
             (r"(^" + custom_tag + r")(.[^\*])",
@@ -412,118 +505,3 @@ Pygments Lexer for the Enterprise Cobol for z/OS and embedded Db2/Cics/DLi featu
                  bygroups(Text.ScrubolTagWarn, Text.ScrubolWarn)),
          ],
     }
-
-if __name__ == "__main__":
-    from pygments import highlight
-    from pygments.formatters import HtmlFormatter
-    from os.path import abspath
-    my_code="""
-cbl rent,data(31),lib                                                           
-       Identification Division.                                                 
-MyTag  Program-id. horror.                                                     
-       Author . datim.                                                          
-       REMARKS. PROGRAMME VRAIMENT HORRIBLE: \tˇ²°²²¹¹¹\t.                                           
-(^w^) *  special cobol tag ! *******************************************70B003
-(^.^) *  special cobol tag ! *******************************************70B003
-(^_^) *  special cobol tag ! *******************************************70B003
-(^x^) *  special cobol tag ! *******************************************70B003
-(^ ^) *  special cobol tag ! *******************************************70B003
-       DATE-WRITTEN. AOUT 2010                                                  
-       Environment Division .
-       Configuration Section .
-       Source-Computer.                                                         
-(^?^)            IBM-370.                                                             
-       Object-computer.                                                         
-           IBM-370.                                                             
-       Special-names .
-           Decimal-point Is Comma .
-       Input-output Section .
-       File-control.
-           Select FFFFFGS Assign  FFFFFGS
-             Organization  Indexed
-             Access Mode  Dynamic 
-             Record Key  FFFFFGS-CLE  
-             File Status  W-FFFFFGS-STATUS  .
-       Data Division.
-       File Section.
-       Fd FFFFFGS 
-MyTag           Label Record  Standard
-MyTag           Data Record  FFFFFGS-ENREG.
-       Copy FFFFFFGS.
-       Working-storage Section.                                                 
-       01 8-9bytes.                                                           
-       01 98-9bytes.                                                           
-''''''      05 four-bytes pic x.                                                           
-pic        03 fives-bytes pic x.                                                           
-       01 filler   Pic  s9(9)  Binary.                                        
-       01 filler-x Pic  s9(9)  Binary.                                        
-       05 ptr4  Redefines full-word  Pointer.                                   
-       05 hexstr          pic X(4) value 'xxxx'.                                
-007220******************************************************************70B003  
-007230*** DéCLARE CURSEUR TABLE MH37=AFFVEAP (C-MH37)                   70B004  
-007240******************************************************************70B005  
-007250     EXEC SQL                                                     COMP.DB2
-''''''       DECLARE C-MH37-U CURSOR   FOR                              COMP.DB2
-007270       SELECT                                                     70B008  
-(^@^)             A.COETBL , A.COTNET                                   70B011  
-007430       FROM AFFVEAP   A                                           COMP.DB2
-MyTag        WHERE                                                      COMP.DB2
-007450*A.COETBL = :MH37-COETBLAND                                       COMP.DB2
-007450 A.COTNET LIKE 'ET%'                                              COMP.DB2
-'''''' A.COTNET > 'ETx'                                                 COMP.DB2
-007520       FOR UPDATE OF                                              70B504  
-007530          COTNET                                                  COMP.DB2
-(^w^) *      OPTIMIZE FOR 1 ROWS                                        COMP.DB2
-007550     END-EXEC.                                                    COMP.DB2
-           01 PFKEY-INDICATOR  VALUE 00  PIC 99.                                
-              88 ENTER-KEY VALUE 00.       88 CLEAR VALUE 93.                   
-              88 PA1   VALUE 92. 88 PA2   VALUE 94. 88 PA3   VALUE 91.          
-              88 PFK1  VALUE 1.  88 PFK2  VALUE 2.  88 PFK3  VALUE 3.           
-       SKIP1                                                                    
-
-000000 Linkage Section.                                                         
-       05 hexstr          pic X(4) value 'xxxx'.                                
-007220******************************************************************70B003
-       05 ptr1   Pointer Occurs 256.                                            
-(^@^)  Procedure Division Using ZC-COMM                                         
-                                TABLE-P       .                                 
-       DANGLING-ELSE.                                                   00394818
-           Exec Cics        
-(^@^)             LINK PROGRAM (ZL00-LCPG8) COMMAREA (ZL99-LICOA) LENGTH        
-MyTag             (ZL00-QLENR)        
-           End-exec.        
-           if cond1 greater or equal 1                                  00394819
-             THEN DISPLAY 'COND1>='                                             
-              if cond2 LESS THAN 10                                     00394819
-                display 'cond2'                                                 
-           else                                                         00394819
-MyTag       display 'else cond1?'                                             
-           else                                                                 
-           Set Address Of cb1 To Null                                           
-         Move  cb1(2:full-word)  To user-name .
-         EXEC DLI GU USING PCB(PCBNUM)
-             SEGMENT((SEGNAME))
-FIX000           KEYS(CONKEYB) KEYLENGTH(8) SETPARENT
-(^.^) * ===>  
-FIX000              SEGMENT(SEGC) INTO(AREAC) SEGLENGTH(SEGLEN)
-(^w^) * ===>  qualified ssa :
-FIX000              WHERE(KEYC=SEGKEYC) FIELDLENGTH(4) END-EXEC.
-         
-"""
-    test='cobol_test.html'
-    highlight(''.join([l.ljust(80) + '\n'
-                       for l in my_code.splitlines()]), # enforce 80 columns
-              IBMCOBOLLexer(encoding='utf-8')
-             ,HtmlFormatter(style=IBMCOBOLStyle
-                            ,encoding='utf-8'
-                            #,noclasses=True
-                            ,full=True
-                            ,cssfile='ibmcob.css'
-                            #,noclobber_cssfile=True
-                            ,linenos='inline'
-                            ,title='Test pygments IBM Cobol Lexer '
-                            ,monofont=True)
-             ,open(test,'wb'))
-    
-    print "file://"+abspath(test),
-    
